import { world, system } from "@minecraft/server";
import './_config_.js';

function score(player, objective) {
    try {
        return world.scoreboard.getObjective(objective).getScore(player.scoreboardIdentity)
    } catch (e) {
        return 0;
    }
}

world.afterEvents.entityDie.subscribe((event) => {
    let entity = event.deadEntity;
    if (entity.typeId === "minecraft:player") {
        entity.runCommand("function player_death")
    }
})

let players_tick = () => {
    for (let player of world.getPlayers()) {
        player.addTag("isUsingSAPI");
        let health = player.getComponent("minecraft:health");
        let currentValue = Math.floor(health.currentValue);
        player.runCommand("scoreboard objectives add health dummy")
        player.runCommand(`scoreboard players set @s health ${currentValue}`)
        "xyz".split("").map(xyz => {
            player.runCommand(`scoreboard objectives add ts:v${xyz} dummy`)
        })
        
        if (currentValue <= 4) {
            player.runCommand("function health_less_4");
        } else {
            player.runCommand("function health_less_4_no");
        }
        
        if (player.isSneaking) {
            player.runCommand("function player_sneaking")
        } else {
            player.runCommand("function player_stop_sneaking")
        }
    }
}

let entities_tick = () => {
    for (const dimension of "overworld|nether|the end".split("|")) {
        let dimensions = world.getDimension(dimension);
        for (const entity of dimensions.getEntities({tags:["tse_st"]})) {
            let tsv = Object.fromEntries("xyz".split("").map(xyz => {
                return [ xyz, score(entity, "ts:v" + xyz) ]
            }))
            entity.runCommand("tp @s[tag=tse,tag=!time_stop_er] @s")
            if (!entity.hasTag("tse_over")) {
                if (!entity.hasTag("tse")) {
                    let velocity = entity.getVelocity()
                    if (entity.typeId !== "minecraft:player") {
                        entity.clearVelocity();
                    }
                    let scoreboardVelocity = Object.fromEntries("xyz".split("").map(xyz => {
                        return [ xyz, parseInt(velocity[xyz]) * Math.pow(10,3) ]
                    }))
                    "xyz".split("").map(xyz => {
                        entity.runCommand(`scoreboard players set @s ts:v${xyz} ${scoreboardVelocity[xyz]}`)
                    })
                    entity.addTag("tse")
                    entity.addTag("unable_skills")
                }
            } else {
                entity.removeTag("tse")
                entity.removeTag("tse_st")
                entity.removeTag("tse_over")
                entity.removeTag("unable_skills")
                if (entity.typeId !== "minecraft:player") {
                    entity.applyImpulse(Object.fromEntries("xyz".split("").map(xyz => {
                        return [ xyz, tsv[xyz] / Math.pow(10,3) ]
                    })));
                }
            }
            if (entity.typeId !== "minecraft:player") {
                entity.applyImpulse(Object.fromEntries("xyz".split("").map(xyz => {
                    return [ xyz, tsv[xyz] / Math.pow(10,8) ]
                })));
            } else {
                entity.teleport(Object.fromEntries("xyz".split("").map(xyz => {
                    return [ xyz, entity.location[xyz] ]
                })))
            }
        }
        for (const entity of dimensions.getEntities({tags:["gd_m_ed"]}).filter(entity => entity.isFalling)) {
            entity.runCommand("damage @s 1")
            entity.runCommand("effect @s fatal_poison 2 0")
            entity.runCommand("playsound gd_m_e @a[r=4] ~~~")
            entity.runCommand("particle jjba:gd_m_e ~~0.5~")
        }
    }
}

system.runInterval(() => {
    players_tick();
    entities_tick();
})
