import { world, system } from "@minecraft/server";

system.runInterval(() => {
    const players = world.getPlayers({ tags: ["cm_l"] });

    players.forEach(player => {
        const entities = player.dimension.getEntities({
            maxDistance: 10,
            location: player.location,
            excludeTypes: ["minecraft:item", "minecraft:xp_orb", "minecraft:armor_stand", "jjba:c_moon"]
        });

        entities.forEach(entity => {
            if (isPlayerLookingAtEntity(player, entity, 10) && !isEntityBehindBlock(player, entity)) {
                if (!entity.hasTag("cm_looked")) {
                    entity.addTag("cm_looked");
                }

                if (entity.hasTag("Dinnerbone")) {
                    entity.nameTag = "Dinnerbone";
                }
            } else {
                removeTags(entity, ["cm_looked"]);

                if (!entity.hasTag("Dinnerbone") && entity.nameTag === "Dinnerbone") {
                    entity.nameTag = "";
                }
            }
        });
    });
});

function isPlayerLookingAtEntity(player, entity, range) {
    const playerPos = player.getHeadLocation();
    const entityPos = entity.location;

    const distance = Math.sqrt(
        Math.pow(entityPos.x - playerPos.x, 2) +
        Math.pow(entityPos.y - playerPos.y, 2) +
        Math.pow(entityPos.z - playerPos.z, 2)
    );

    if (distance > range) {
        return false;
    }

    const viewDirection = player.getViewDirection();

    const directionToEntity = {
        x: entityPos.x - playerPos.x,
        y: entityPos.y - playerPos.y,
        z: entityPos.z - playerPos.z
    };

    const directionToEntityNormalized = normalizeVector(directionToEntity);

    const dotProduct = directionToEntityNormalized.x * viewDirection.x +
        directionToEntityNormalized.y * viewDirection.y +
        directionToEntityNormalized.z * viewDirection.z;

    return dotProduct > 0.65;
}

function isEntityBehindBlock(player, entity) {
    const playerPos = player.getHeadLocation();
    const entityPos = entity.location;

    const deltaX = entityPos.x - playerPos.x;
    const deltaY = entityPos.y - playerPos.y;
    const deltaZ = entityPos.z - playerPos.z;

    const distance = Math.sqrt(deltaX * deltaX + deltaY * deltaY + deltaZ * deltaZ);

    const steps = Math.ceil(distance * 20);
    const stepX = deltaX / steps;
    const stepY = deltaY / steps;
    const stepZ = deltaZ / steps;

    let currentX = playerPos.x;
    let currentY = playerPos.y;
    let currentZ = playerPos.z;

    for (let i = 0; i < steps; i++) {
        currentX += stepX;
        currentY += stepY;
        currentZ += stepZ;

        const block = player.dimension.getBlock({
            x: Math.floor(currentX),
            y: Math.floor(currentY),
            z: Math.floor(currentZ)
        });

        if (block && block.typeId !== "minecraft:air") {
            return true;
        }
    }

    return false;
}

function removeTags(entity, tags) {
    tags.forEach(tag => {
        if (entity.hasTag(tag)) {
            entity.removeTag(tag);
        }
    });
}

function normalizeVector(vector) {
    const length = Math.sqrt(vector.x * vector.x + vector.y * vector.y + vector.z * vector.z);
    return {
        x: vector.x / length,
        y: vector.y / length,
        z: vector.z / length
    };
}
