import { world, ItemStack } from "@minecraft/server";

function playSound(player, sound, radius = 1) {
    player.dimension.runCommandAsync(`playsound ${sound} @a[r=${radius}] ~ ~ ~`);
}

function applyEffects(player, effects) {
    effects.forEach(({ effect, duration, amplifier, hideParticles = true }) => {
        player.addEffect(effect, duration * 20, { amplifier: amplifier - 1, showParticles: !hideParticles });
    });
}

function runCommands(player, commands) {
    commands.forEach(command => player.runCommandAsync(command));
}

function showParticles(player, particle) {
    player.dimension.runCommandAsync(`particle ${particle} ~ ~1 ~`);
}

function clearAllEffects(player) {
    player.dimension.runCommandAsync(`effect @s clear`);
}

const customFoodEffects = [
    {
        itemId: "jjba:jambalaya",
        commands: [`playsound jambalaya @a[r=1] ~ ~ ~`],
    },
    {
        itemId: "jjba:spaghetti",
        effects: [
            { effect: "regeneration", duration: 10, amplifier: 1 },
            { effect: "strength", duration: 10, amplifier: 1 },
        ],
        commands: [
            `tag @p remove pjeffect`,
            `particle nika:cake ~ ~1 ~`,
        ],
    },
    {
        itemId: "jjba:cookie",
        effects: [
            { effect: "speed", duration: 3, amplifier: 1 },
            { effect: "slowness", duration: 3, amplifier: 1 },
            { effect: "haste", duration: 3, amplifier: 1 },
            { effect: "mining_fatigue", duration: 3, amplifier: 1 },
            { effect: "strength", duration: 3, amplifier: 1 },
            { effect: "instant_health", duration: 3, amplifier: 1 },
            { effect: "instant_damage", duration: 3, amplifier: 1 },
            { effect: "jump_boost", duration: 3, amplifier: 1 },
            { effect: "nausea", duration: 3, amplifier: 1 },
            { effect: "regeneration", duration: 3, amplifier: 1 },
            { effect: "resistance", duration: 3, amplifier: 1 },
            { effect: "fire_resistance", duration: 3, amplifier: 1 },
            { effect: "water_breathing", duration: 3, amplifier: 1 },
            { effect: "invisibility", duration: 3, amplifier: 1 },
            { effect: "blindness", duration: 3, amplifier: 1 },
            { effect: "night_vision", duration: 3, amplifier: 1 },
            { effect: "hunger", duration: 3, amplifier: 1 },
            { effect: "weakness", duration: 3, amplifier: 1 },
            { effect: "poison", duration: 3, amplifier: 1 },
            { effect: "wither", duration: 3, amplifier: 1 },
            { effect: "health_boost", duration: 3, amplifier: 1 },
            { effect: "absorption", duration: 3, amplifier: 1 },
            { effect: "saturation", duration: 3, amplifier: 1 },
            { effect: "levitation", duration: 3, amplifier: 1 },
            { effect: "slow_falling", duration: 3, amplifier: 1 },
            { effect: "bad_omen", duration: 3, amplifier: 1 },
            { effect: "village_hero", duration: 3, amplifier: 1 },
        ],
    },
    {
        itemId: "jjba:pizza",
        effects: [{ effect: "speed", duration: 5, amplifier: 1 }],
    },
    {
        itemId: "jjba:waterbottle",
        effects: [
            { effect: "night_vision", duration: 120, amplifier: 20 },
            { effect: "slowness", duration: 4, amplifier: 255 },
            { effect: "blindness", duration: 4, amplifier: 4 },
        ],
        commands: [
            `playanimation @p animation.water`,
            `particle nika:water ^ ^0.25 ^0.22`,
            `playsound water @a[r=2] ~ ~ ~`,
        ],
    },
    {
        itemId: "jjba:cake",
        functions: [clearAllEffects],
    },
    {
        itemId: "jjba:bsb1",
        commands: [`execute as @s at @s run function bsb1`],
    },
    {
        itemId: "jjba:bsb2",
        commands: [`execute as @s at @s run function bsb2`],
    },
];

world.afterEvents.itemCompleteUse.subscribe(({ itemStack: item, source: player }) => {
    const foodEffect = customFoodEffects.find(food => food.itemId === item.typeId);
    if (foodEffect) {
        const { effects, commands, functions } = foodEffect;
        if (effects) applyEffects(player, effects);
        if (commands) runCommands(player, commands);
        if (functions) {
            functions.forEach(func => {
                if (func.length === 2) {
                    func(player, item);
                } else {
                    func(player);
                }
            });
        }
    }
});
