import { world, system } from "@minecraft/server";

function spawnProjectile(player, projectileType, speed = 1.5) {
    const playerLocation = player.location;
    const playerDirection = player.getViewDirection();
    const dimension = player.dimension;

    const spawnLocation = {
        x: playerLocation.x + playerDirection.x,
        y: playerLocation.y + playerDirection.y + 1.5,
        z: playerLocation.z + playerDirection.z
    };

    const projectile = dimension.spawnEntity(projectileType, spawnLocation);

    if (projectile) {
        const impulse = {
            x: playerDirection.x * speed,
            y: playerDirection.y * speed,
            z: playerDirection.z * speed
        };

        projectile.applyImpulse(impulse);

        const intervalId = system.runInterval(() => {
            if (projectile.isValid()) {
                projectile.applyImpulse(impulse);
            } else {
                system.clearRun(intervalId);
            }
        }, 1.5);
    }
}

world.beforeEvents.worldInitialize.subscribe(initEvent => {
    initEvent.itemComponentRegistry.registerCustomComponent('jjba:use_skillger', {
        onUse(e) {
            const player = e.source;
            const item = e.itemStack;

            item.getComponent('minecraft:cooldown').startCooldown(player);


            switch (item.typeId) {
                case 'jjba:ger_stand_disc':
                    player.runCommandAsync('execute as @s[tag=!stand_user,tag=!ge_user] at @s run function ger_stand_up');
                    player.runCommandAsync('effect @s[tag=!stand_user,tag=!ge_user] night_vision 2 10 true');
                    player.runCommandAsync('effect @s[tag=!stand_user,tag=!ge_user] blindness 1 10');
                    break;
                case 'jjba:ge_pb_control1':
                    player.runCommandAsync('execute as @s[tag=!tse,tag=!unable_skills] at @s run function ger_pb_active');
                    break;
                case 'jjba:t1_nail1':
                    player.runCommandAsync('execute as @s[tag=ger_user,tag=!unable_skills] at @s run function beam');
                    spawnProjectile(player, 'jjba:t1_nail1', 1.5);
                    break;
                case 'jjba:ge_gflg1':
                    player.runCommandAsync('execute as @s[tag=ger_user,tag=!tse,tag=!shift,tag=!unable_skills] at @s run function ger_gflg');
                    player.runCommandAsync('execute as @s[tag=ger_user,tag=shift,tag=!tse,tag=!unable_skills] at @s run function ger_lf');
                    break;
                case 'jjba:ge_disabled_control1':
                    player.runCommandAsync('execute as @s[tag=ger_user,tag=!tse,tag=!rt] run function ger_disabled');
                    break;
                case 'jjba:ger_cure':
                    player.runCommandAsync('execute as @s[tag=!tse] at @s run function ger_cure');
                    break;
                case 'jjba:ge_active_control1':
                    player.runCommandAsync('execute as @s[tag=ger_user,tag=!tse] at @s run function ger_active');
                    break;
                case 'jjba:ger_control':
                    player.runCommandAsync('execute as @s[tag=ger_user,tag=!tse,tag=!unable_skills] at @s run function ger0');
                    break;
                case 'jjba:ger_cc':
                    player.runCommandAsync('function ger_cc');
                    break;
                default:
                    return;
            }
        }
    });
});
